import pytest

## ==========================

def test_numpy_import():
    import numpy
    return

def test_scipy_import():
    import scipy
    print("\t\t You have scipy version {}".format(scipy.__version__))


def test_stripy_modules():
    import stripy
    from stripy import documentation
    from stripy import spherical_meshes
    from stripy import cartesian_meshes
    from stripy import sTriangulation
    from stripy import Triangulation


def test_jupyter_available():
    from subprocess import check_output
    try:
        result = str(check_output(['which', 'jupyter']))[2:-3]
    except:
        print("Jupyter not installed")
        print("Jupyter is needed to run the example documentation")

# def test_documentation_dependencies():
#     import matplotlib
#     import cartopy
#     import imageio
#     import lavavu
#     import pyproj
#
# def test_litho1pt0():
#     import litho1pt0



import pytest
import stripy

# These tests just test the mesh construction doesn't fail at the moment
# ToDo: add meaningful assertions

# It's hard to parameterise these as the calls are a little
# bit different in each case

def test_kd_tree_building():
    mesh = stripy.spherical_meshes.icosahedral_mesh(include_face_points=True, refinement_levels=2, tree=True)


def test_triangulated_cube_mesh():

    mesh = stripy.spherical_meshes.triangulated_cube_mesh(refinement_levels=0)
    mesh = stripy.spherical_meshes.triangulated_cube_mesh(refinement_levels=2)


def test_octahedral_mesh():

    mesh = stripy.spherical_meshes.octahedral_mesh(include_face_points=False, refinement_levels=0)
    mesh = stripy.spherical_meshes.octahedral_mesh(include_face_points=False, refinement_levels=2)
    mesh = stripy.spherical_meshes.octahedral_mesh(include_face_points=True,  refinement_levels=0)
    mesh = stripy.spherical_meshes.octahedral_mesh(include_face_points=True,  refinement_levels=2)

def test_icosahedral_mesh():

    mesh = stripy.spherical_meshes.icosahedral_mesh(include_face_points=False, refinement_levels=0)
    mesh = stripy.spherical_meshes.icosahedral_mesh(include_face_points=False, refinement_levels=2)
    mesh = stripy.spherical_meshes.icosahedral_mesh(include_face_points=True,  refinement_levels=0)
    mesh = stripy.spherical_meshes.icosahedral_mesh(include_face_points=True,  refinement_levels=2)


def test_triangulated_soccerball_mesh():

    mesh = stripy.spherical_meshes.triangulated_soccerball_mesh(refinement_levels=0)
    mesh = stripy.spherical_meshes.triangulated_soccerball_mesh(refinement_levels=2)

def test_random_mesh():

    mesh = stripy.spherical_meshes.random_mesh(number_of_points=10)
    mesh = stripy.spherical_meshes.random_mesh(number_of_points=10000)


def test_uniform_ring_mesh():

    mesh = stripy.spherical_meshes.uniform_ring_mesh(resolution=9, refinement_levels=0)
    mesh = stripy.spherical_meshes.uniform_ring_mesh(resolution=3, refinement_levels=3)




import pytest
import stripy
import numpy as np

@pytest.mark.parametrize("permute", [False, True])
def test_nearest_nd_interpolation(permute):

    coords = np.array([[0.0, 0.0+np.random.random()*1.0e-16], 
                       [0.0, 1.0+np.random.random()*1.0e-16], 
                       [1.0, 0.0+np.random.random()*1.0e-16], 
                       [1.0, 1.0+np.random.random()*1.0e-16]])

    x, y = coords[:,0], coords[:,1]
    mesh = stripy.Triangulation(x, y, permute=permute)

    Z = np.linspace(0.0, 10.0, mesh.npoints)

    ix = x[0] + 0.001
    iy = y[0] + 0.001

    Zi, ierr = mesh.interpolate_nearest(ix, iy, Z)

    if Zi == Z[0]:
        print("PASS! (Interpolation - nearest neighbour)")
    else:
        assert False, "FAIL! (Interpolation - nearest neighbour)"


@pytest.mark.parametrize("permute", [False, True])
def test_linear_interpolation(permute):

    coords = np.array([[0.0, 0.0+np.random.random()*1.0e-16], 
                       [0.0, 1.0+np.random.random()*1.0e-16], 
                       [1.0, 0.0+np.random.random()*1.0e-16], 
                       [1.0, 1.0+np.random.random()*1.0e-16], 
                       [0.5, 0.5+np.random.random()*1.0e-16]])

    x, y = coords[:,0], coords[:,1]
    mesh = stripy.Triangulation(x, y, permute=permute)

    Z = mesh.x

    npts = 5
    ix = np.linspace(0.0, 1.0, npts)
    iy = np.zeros(npts)

    Zi, ierr = mesh.interpolate_linear(ix, iy, Z)

    # this should be true
    # but machine precision may differ so we don't test it
    # print((Zi == ix).all())

    bounded = Zi[0] == ix[0] and Zi[-1] == ix[-1]
    ascending = ( np.diff(Zi) > 0 ).all()

    if bounded and ascending:
        print("PASS! (Interpolation - linear")
    else:
        assert False, "FAIL! (Interpolation - linear)"


@pytest.mark.parametrize("permute", [False, True])
def test_cubic_interpolation(permute):

    # we need more points for cubic interpolation
    coords = np.array([[0.0, 0.0+np.random.random()*1.0e-16], 
                       [0.0, 1.0+np.random.random()*1.0e-16], 
                       [1.0, 0.0+np.random.random()*1.0e-16], 
                       [1.0, 1.0+np.random.random()*1.0e-16], 
                       [0.1, 0.1+np.random.random()*1.0e-16], 
                       [0.1, 0.9+np.random.random()*1.0e-16], 
                       [0.9, 0.1+np.random.random()*1.0e-16], 
                       [0.9, 0.9+np.random.random()*1.0e-16]])

    x, y = coords[:,0], coords[:,1]
    mesh = stripy.Triangulation(x, y, permute=permute)

    Z = mesh.x**2

    npts = 7
    xi = np.linspace(0.0, 1.0, npts)
    yi = np.zeros(npts)

    Zi_linear, ierr = mesh.interpolate_linear(xi, yi, Z)
    Zi_cubic,  ierr = mesh.interpolate_cubic(xi, yi, Z)

    diff_linear = np.abs(Zi_linear - xi**2).sum()
    diff_cubic  = np.abs(Zi_cubic  - xi**2).sum()

    # check if cubic interpolation is more accurate than linear
    if diff_cubic < diff_linear:
        print("PASS! (Interpolation - cubic")
    else:
        assert False, "FAIL! (Interpolation - cubic)"


@pytest.mark.parametrize("permute", [False, True])
def test_cubic_interpolation_tension(permute):

    # we need more points for cubic interpolation
    coords = np.array([[0.0, 0.0+np.random.random()*1.0e-16], 
                       [0.0, 1.0+np.random.random()*1.0e-16], 
                       [1.0, 0.0+np.random.random()*1.0e-16], 
                       [1.0, 1.0+np.random.random()*1.0e-16], 
                       [0.1, 0.1+np.random.random()*1.0e-16], 
                       [0.1, 0.9+np.random.random()*1.0e-16], 
                       [0.9, 0.1+np.random.random()*1.0e-16], 
                       [0.9, 0.9+np.random.random()*1.0e-16]])

    x, y = coords[:,0], coords[:,1]


    mesh = stripy.Triangulation(x, y, permute=permute)

    Z = mesh.x**2

    npts = 7
    xi = np.linspace(0.0, 1.0, npts)
    yi = np.zeros(npts)

    Zi_linear, ierr = mesh.interpolate_linear(xi, yi, Z)
    Zi_cubic,  ierr = mesh.interpolate_cubic(xi, yi, Z)

    sigma = mesh.get_spline_tension_factors(Z)
    Zi_cubicT, ierr = mesh.interpolate_cubic(xi, yi, Z, sigma=sigma)

    sigma.fill(45.)
    Zi_cubicTmax, ierr = mesh.interpolate_cubic(xi, yi, Z, sigma=sigma)

    diff_linear = np.abs(Zi_linear - xi**2).sum()
    diff_cubic  = np.abs(Zi_cubic  - xi**2).sum()


    if np.abs(Zi_cubicT - Zi_cubic).any():
        print("PASS! (Interpolation - cubic tensioned splines")
    # check if cubic interpolation with max tension is like linear interpolation
    elif np.abs(Zi_linear-Zi_cubicTmax).sum() < np.abs(Zi_cubic-Zi_cubicTmax).sum():
        print("PASS! (Interpolation - cubic tensioned splines")
    else:
        assert False, "FAIL! (Interpolation - cubic tensioned splines)"


@pytest.mark.parametrize("permute", [False])
def test_cubic_interpolation_grid(permute):

    # we need more points for cubic interpolation
    coords = np.array([[0.0, 0.0+np.random.random()*1.0e-16], 
                       [0.0, 1.0+np.random.random()*1.0e-16], 
                       [1.0, 0.0+np.random.random()*1.0e-16], 
                       [1.0, 1.0+np.random.random()*1.0e-16], 
                       [0.1, 0.1+np.random.random()*1.0e-16], 
                       [0.1, 0.9+np.random.random()*1.0e-16], 
                       [0.9, 0.1+np.random.random()*1.0e-16], 
                       [0.9, 0.9+np.random.random()*1.0e-16]])

    x, y = coords[:,0], coords[:,1]
    mesh = stripy.Triangulation(x, y, permute=permute)

    Z = mesh.x**2

    npts = 7
    xi = np.linspace(-0.1, 1.1, npts)
    yi = np.linspace(-0.1, 1.1, npts)
    xq, yq = np.meshgrid(xi,yi)
    shape = (npts, npts)

    Zi_cubic,  ierr = mesh.interpolate_cubic(xq.ravel(), yq.ravel(), Z)
    Zi_cubic_grid = mesh.interpolate_to_grid(xi, yi, Z)

    sigma = mesh.get_spline_tension_factors(Z, tol=1e-5)
    Zi_cubic_grid_S = mesh.interpolate_to_grid(xi, yi, Z, sigma=sigma)

    err_msg = "Interpolate to grid - cubic tensioned splines"
    np.testing.assert_allclose(Zi_cubic.reshape(shape), Zi_cubic_grid, atol=0.1, err_msg=err_msg)
    np.testing.assert_allclose(Zi_cubic_grid_S, Zi_cubic_grid, atol=0.5, err_msg=err_msg)
    
    assert (Zi_cubic_grid_S != Zi_cubic_grid).any(), err_msg


@pytest.mark.parametrize("permute", [False, True])
def test_derivative(permute):

    p0 = 0.0
    p1 = 1.0
    p2 = 2.0

    coords = np.array([[p0 , -p2+np.random.random()*1.0e-16], 
                       [-p2,  p0+np.random.random()*1.0e-16], 
                       [p0 ,  p2+np.random.random()*1.0e-16], 
                       [p2 ,  p0+np.random.random()*1.0e-16], 
                       [p0 , -p1+np.random.random()*1.0e-16], 
                       [-p1,  p0+np.random.random()*1.0e-16], 
                       [p0 ,  p1+np.random.random()*1.0e-16], 
                       [p1 ,  p0+np.random.random()*1.0e-16], 
                       [-p1, -p1+np.random.random()*1.0e-16], 
                       [-p1,  p1+np.random.random()*1.0e-16], 
                       [p1 ,  p1+np.random.random()*1.0e-16], 
                       [p1 , -p1+np.random.random()*1.0e-16], 
                       [p0 ,  p0+np.random.random()*1.0e-16]])

    x, y = coords[:,0], coords[:,1]
    mesh = stripy.Triangulation(x, y, permute=permute)

    # create a soup bowl
    Z = mesh.x**2 + mesh.y**2

    # derivatives will have a constant gradient
    dZdx, dZdy = mesh.gradient(Z, nit=10, tol=1e-12)

    # interpolate onto a straight line
    ipts = np.linspace(-p2, p2, 5)
    dZdx_interp, ierr = mesh.interpolate_linear(ipts, ipts*0, dZdx)
    dZdy_interp, ierr = mesh.interpolate_linear(ipts*0, ipts, dZdy)

    ascending_xi = ( np.diff(dZdx_interp) > 0 ).all()
    ascending_yi = ( np.diff(dZdy_interp) > 0 ).all()

    if ascending_xi and ascending_yi:
        print("PASS! (Derivatives)")
    else:
        assert False, "FAIL! (Derivatives)"


@pytest.mark.parametrize("permute", [False, True])
def test_smoothing(permute):

    # we need more points for cubic interpolation
    coords = np.array([[0.0, 0.0+np.random.random()*1.0e-16],
                       [0.0, 1.0+np.random.random()*1.0e-16],
                       [1.0, 0.0+np.random.random()*1.0e-16],
                       [1.0, 1.0+np.random.random()*1.0e-16],
                       [0.1, 0.1+np.random.random()*1.0e-16],
                       [0.1, 0.9+np.random.random()*1.0e-16], 
                       [0.9, 0.1+np.random.random()*1.0e-16], 
                       [0.9, 0.9+np.random.random()*1.0e-16]])

    x, y = coords[:,0], coords[:,1]
    mesh = stripy.Triangulation(x, y, permute=permute)

    Z = np.ones_like(x)
    Z[-1] = 0.0

    weights = np.ones_like(x)
    f_smooth, f_derivatives, ierr = mesh.smoothing(Z, weights, 0.05, 1e-2, 1e-5)

    if (f_smooth.max() - f_smooth.min()) < 1.0:
        print("PASS! (Smoothing)")
    else:
        assert False, "FAIL! (Smoothing)"


if __name__ == "__main__":
    test_derivative()
    test_nearest_nd_interpolation()
    test_linear_interpolation()
    test_cubic_interpolation()
    test_cubic_interpolation_tension()
    test_cubic_interpolation_grid()
    test_smoothing()



import pytest
import stripy
import numpy as np

@pytest.mark.parametrize("permute", [False, True])
def test_nearest_nd_interpolation(permute):

    p0 = 0.0
    p2 = np.pi/2
    p4 = np.pi/4

    coords = np.array([[p0 , -p2+np.random.random()*1.0e-16], \
                       [-p2,  p0+np.random.random()*1.0e-16], \
                       [p0 ,  p2+np.random.random()*1.0e-16], \
                       [p2 ,  p0+np.random.random()*1.0e-16]])

    lons, lats = coords[:,0], coords[:,1]
    mesh = stripy.sTriangulation(lons, lats, permute=permute)

    Z = np.linspace(-p2, p2, mesh.npoints)

    ilons = lons[0] + 0.001
    ilats = lats[0] + 0.001

    Zi, ierr = mesh.interpolate_nearest(ilons, ilats, Z)

    # check if we return the nearest numbers
    if Zi == Z[0]:
        print("PASS! (Interpolation - nearest neighbour)")
    else:
        assert False, "FAIL! (Interpolation - nearest neighbour)"


@pytest.mark.parametrize("permute", [False, True])
def test_linear_interpolation(permute):

    p0 = 0.0
    p2 = np.pi/2
    p4 = np.pi/4

    coords = np.array([[p0 , -p2+np.random.random()*1.0e-16], \
                       [-p2,  p0+np.random.random()*1.0e-16], \
                       [p0 ,  p2+np.random.random()*1.0e-16], \
                       [p2 ,  p0+np.random.random()*1.0e-16], \
                       [p0 ,  p0+np.random.random()*1.0e-16]])

    lons, lats = coords[:,0], coords[:,1]
    mesh = stripy.sTriangulation(lons, lats, permute=permute)

    Z = mesh.lons

    npts = 5
    ilons = np.linspace(-p2, p2, npts)
    ilats = np.zeros(npts)

    Zi, ierr = mesh.interpolate_linear(ilons, ilats, Z)

    # this should be true
    # but machine precision may differ so we don't test it
    # print((Zi == ilons).all())

    bounded = Zi[0] == ilons[0] and Zi[-1] == ilons[-1]
    ascending = ( np.diff(Zi) > 0 ).all()

    # check if linear gradient across the equator
    if bounded and ascending:
        print("PASS! (Interpolation - linear")
    else:
        assert False, "FAIL! (Interpolation - linear)"


@pytest.mark.parametrize("permute", [False, True])
def test_cubic_interpolation(permute):

    p0 = 0.0
    p2 = np.pi/2
    p4 = np.pi/4

    # we need more points for cubic interpolation
    coords = np.array([[p0 , -p2+np.random.random()*1.0e-16], \
                       [-p2,  p0+np.random.random()*1.0e-16], \
                       [p0 ,  p2+np.random.random()*1.0e-16], \
                       [p2 ,  p0+np.random.random()*1.0e-16], \
                       [p0 , -p4+np.random.random()*1.0e-16], \
                       [-p4,  p0+np.random.random()*1.0e-16], \
                       [p0 ,  p4+np.random.random()*1.0e-16], \
                       [p4 ,  p0+np.random.random()*1.0e-16]])

    lons, lats = coords[:,0], coords[:,1]
    mesh = stripy.sTriangulation(lons, lats, permute=permute)

    Z = mesh.lons**2

    npts = 7
    ilons = np.linspace(-p2, p2, npts)
    ilats = np.zeros(npts)

    Zi_linear, ierr = mesh.interpolate_linear(ilons, ilats, Z)
    Zi_cubic,  ierr = mesh.interpolate_cubic(ilons, ilats, Z)

    diff_linear = np.abs(Zi_linear - ilons**2).sum()
    diff_cubic  = np.abs(Zi_cubic  - ilons**2).sum()

    # check if cubic interpolation is more accurate than linear
    if diff_cubic < diff_linear:
        print("PASS! (Interpolation - cubic")
    else:
        assert False, "FAIL! (Interpolation - cubic)"


@pytest.mark.parametrize("permute", [False, True])
def test_cubic_interpolation_tension(permute):

    p0 = 0.0
    p2 = np.pi/2
    p4 = np.pi/4

    # we need more points for cubic interpolation
    coords = np.array([[p0 , -p2+np.random.random()*1.0e-16], \
                       [-p2,  p0+np.random.random()*1.0e-16], \
                       [p0 ,  p2+np.random.random()*1.0e-16], \
                       [p2 ,  p0+np.random.random()*1.0e-16], \
                       [p0 , -p4+np.random.random()*1.0e-16], \
                       [-p4,  p0+np.random.random()*1.0e-16], \
                       [p0 ,  p4+np.random.random()*1.0e-16], \
                       [p4 ,  p0+np.random.random()*1.0e-16]])

    lons, lats = coords[:,0], coords[:,1]
    mesh = stripy.sTriangulation(lons, lats, permute=permute)

    Z = mesh.lons**2

    npts = 7
    ilons = np.linspace(-p2, p2, npts)
    ilats = np.zeros(npts)

    Zi_linear, ierr = mesh.interpolate_linear(ilons, ilats, Z)
    Zi_cubic,  ierr = mesh.interpolate_cubic(ilons, ilats, Z)

    sigma = mesh.get_spline_tension_factors(Z)
    Zi_cubicT,  ierr = mesh.interpolate_cubic(ilons, ilats, Z, sigma=sigma)

    sigma.fill(45.)
    Zi_cubicTmax,  ierr = mesh.interpolate_cubic(ilons, ilats, Z, sigma=sigma)


    if np.abs(Zi_cubicT - Zi_cubic).any():
        print("PASS! (Interpolation - cubic tensioned splines")
    # check if cubic interpolation with max tension is like linear interpolation
    elif np.abs(Zi_linear-Zi_cubicTmax).sum() < np.abs(Zi_cubic-Zi_cubicTmax).sum():
        print("PASS! (Interpolation - cubic tensioned splines")
    else:
        assert False, "FAIL! (Interpolation - cubic tensioned splines)"


@pytest.mark.parametrize("permute", [False])
def test_cubic_interpolation_grid(permute):

    p0 = 0.0
    p2 = np.pi/2
    p4 = np.pi/4

    # we need more points for cubic interpolation
    coords = np.array([[p0 , -p2+np.random.random()*1.0e-16], \
                       [-p2,  p0+np.random.random()*1.0e-16], \
                       [p0 ,  p2+np.random.random()*1.0e-16], \
                       [p2 ,  p0+np.random.random()*1.0e-16], \
                       [p0 , -p4+np.random.random()*1.0e-16], \
                       [-p4,  p0+np.random.random()*1.0e-16], \
                       [p0 ,  p4+np.random.random()*1.0e-16], \
                       [p4 ,  p0+np.random.random()*1.0e-16]])

    lons, lats = coords[:,0], coords[:,1]
    mesh = stripy.sTriangulation(lons, lats, permute=permute)

    Z = mesh.lons**2

    npts = 7
    ilons = np.linspace(-p2, p2, npts)
    ilats = np.linspace(-p2, p2, npts)
    lonq, latq = np.meshgrid(ilons, ilats)
    shape = (npts, npts)

    Zi_cubic, zierr = mesh.interpolate_cubic(lonq.ravel(), latq.ravel(), Z)
    Zi_cubic_grid   = mesh.interpolate_to_grid(ilons, ilats, Z)

    sigma = mesh.get_spline_tension_factors(Z, tol=1e-5)
    Zi_cubic_grid_S = mesh.interpolate_to_grid(ilons, ilats, Z, sigma=sigma)

    err_msg = "Interpolate to grid - cubic tensioned splines"

    np.testing.assert_allclose(Zi_cubic.reshape(shape), Zi_cubic_grid, atol=0.1, err_msg=err_msg)
    np.testing.assert_allclose(Zi_cubic_grid_S, Zi_cubic_grid, atol=0.5, err_msg=err_msg)

    # This has an unpredictable error in it that is causing trouble
    # for all of our automated tests and conda builds:

    assert (Zi_cubic_grid_S != Zi_cubic_grid).any(), err_msg

    # unstructured and grid interpolation works
    # and applying tension alters the result


@pytest.mark.parametrize("permute", [False, True])
def test_derivative(permute):

    p0 = 0.0
    p2 = np.pi/2
    p4 = np.pi/4

    coords = np.array([[p0 , -p2+np.random.random()*1.0e-16], 
                       [-p2,  p0+np.random.random()*1.0e-16], 
                       [p0 ,  p2+np.random.random()*1.0e-16], 
                       [p2 ,  p0+np.random.random()*1.0e-16], 
                       [p0 , -p4+np.random.random()*1.0e-16], 
                       [-p4,  p0+np.random.random()*1.0e-16], 
                       [p0 ,  p4+np.random.random()*1.0e-16], 
                       [p4 ,  p0+np.random.random()*1.0e-16], 
                       [-p4, -p4+np.random.random()*1.0e-16], 
                       [-p4,  p4+np.random.random()*1.0e-16], 
                       [p4 ,  p4+np.random.random()*1.0e-16], 
                       [p4 , -p4+np.random.random()*1.0e-16], 
                       [p0 ,  p0+np.random.random()*1.0e-16]])

    lons, lats = coords[:,0], coords[:,1]
    mesh = stripy.sTriangulation(lons, lats, permute=permute)

    # create a soup bowl
    Z = mesh.lons**2 + mesh.lats**2

    # derivatives will have a constant gradient
    dZdlon, dZdlat = mesh.gradient_lonlat(Z, nit=10, tol=1e-12)

    # interpolate onto a straight line
    ipts = np.linspace(-p2, p2, 5)
    dZdlon_interp, ierr = mesh.interpolate_linear(ipts, ipts*0, dZdlon)
    dZdlat_interp, ierr = mesh.interpolate_linear(ipts*0, ipts, dZdlat)

    ascending_lon = ( np.diff(dZdlon_interp) > 1.0e-16 ).all()
    ascending_lat = ( np.diff(dZdlat_interp) > 1.0e-16 ).all()

    if ascending_lon and ascending_lat:
        print("PASS! (Derivatives)")
    else:
        assert False, "FAIL! (Derivatives)"


@pytest.mark.parametrize("permute", [False, True])
def test_smoothing(permute):

    p0 = 0.0
    p2 = np.pi/2
    p4 = np.pi/4

    # we need more points for cubic interpolation
    coords = np.array([[p0 , -p2+np.random.random()*1.0e-16], \
                       [-p2,  p0+np.random.random()*1.0e-16], \
                       [p0 ,  p2+np.random.random()*1.0e-16], \
                       [p2 ,  p0+np.random.random()*1.0e-16], \
                       [p0 , -p4+np.random.random()*1.0e-16], \
                       [-p4,  p0+np.random.random()*1.0e-16], \
                       [p0 ,  p4+np.random.random()*1.0e-16], \
                       [p4 ,  p0+np.random.random()*1.0e-16]])

    lons, lats = coords[:,0], coords[:,1]
    mesh = stripy.sTriangulation(lons, lats, permute=permute)

    Z = np.ones_like(lons)
    Z[-1] = 0.0

    weights = np.ones_like(lons)
    f_smooth, f_derivatives, ierr = mesh.smoothing(Z, weights, 0.05, 1e-2, 1e-5)

    # check if f_smooth is smoother than f
    if (f_smooth.max() - f_smooth.min()) < 1.0:
        print("PASS! (Smoothing)")
    else:
        assert False, "FAIL! (Smoothing)"


if __name__ == "__main__":
    test_derivative()
    test_nearest_nd_interpolation()
    test_linear_interpolation()
    test_cubic_interpolation()
    test_cubic_interpolation_tension()
    test_cubic_interpolation_grid()
    test_smoothing()



import numpy as np
import stripy
from stripy import cartesian_meshes
from stripy import spherical_meshes
from time import time

# global variables
permute = False
max_refinements = 2
npoints = 10000


def time_routine(routine, *args):
    t = time()
    routine(*args)
    str_fmt = "- {:30} completed in  {:1.6f} seconds"
    print(str_fmt.format(str(routine.__name__), time()-t))

"""
Test Triangulation routines
"""


def test_cartesian_triangulation():
    # Create some (semi) random points
    np.random.seed(0)
    x = np.random.random(npoints)
    y = np.random.random(npoints)

    # interpolation data
    xi = np.random.random(100)
    yi = np.random.random(100)
    rint = np.random.randint(0, len(xi), len(xi))

    # triangulation
    tri = stripy.Triangulation(x, y)
    assert tri.npoints == npoints
    return(tri, x, y, xi, yi)


def test_cartesian_refinement():
    tri, x, y, xi, yi = test_cartesian_triangulation()

    for refine in range(max_refinements):
        print("\nrefinement = {}\n".format(refine))
        time_routine(tri.__init__, x, y, refine, permute)

        z = np.hypot(tri.x, tri.y)

        time_routine(tri.areas)
        time_routine(tri.convex_hull)
        time_routine(tri.gradient, z)
        time_routine(tri.gradient_local, z, 0)
        time_routine(tri.interpolate_nearest, xi, yi, z)
        time_routine(tri.interpolate_linear, xi, yi, z)
        time_routine(tri.interpolate_cubic, xi, yi, z)
        time_routine(tri.nearest_vertex, xi, yi)
        time_routine(tri.containing_triangle, xi, yi)
        time_routine(tri.containing_simplex_and_bcc, xi, yi)

    return

def test_spherical_triangulation():
    # interpolation data
    lon = 2.*np.pi*np.random.random(100)
    lat = np.arccos(2.*np.random.random(100) - 1.) - np.pi/2

    # stri = stripy.sTriangulation(lons, lats)
    stri = stripy.spherical_meshes.octahedral_mesh(include_face_points=True, refinement_levels=3)
    stri = stripy.sTriangulation(stri.lons, stri.lats)

    return(stri, stri.lons, stri.lats, lon, lat)

def test_spherical_refinement():

    stri, lons, lats, lon, lat = test_spherical_triangulation()
    rint = np.random.randint(0, len(lon), len(lon))

    for refine in range(max_refinements):
        print("\nrefinement = {}\n".format(refine))
        time_routine(stri.__init__, stri.lons, stri.lats, refine, permute)

        Z = np.cos(5.0*stri.lons) * np.sin(2.0*stri.lats)

        time_routine(stri.areas)
        time_routine(stri.gradient_lonlat, Z)
        time_routine(stri.interpolate_nearest, lon, lat, Z)
        time_routine(stri.interpolate_linear, lon, lat, Z)
        time_routine(stri.interpolate_cubic, lon, lat, Z)
        time_routine(stri.nearest_vertex, lon, lat)
        time_routine(stri.tri_area, lon[:3], lat[:3])
        time_routine(stri.identify_vertex_neighbours, rint[0])
        time_routine(stri.containing_triangle, lon, lat)
        time_routine(stri.containing_simplex_and_bcc, lon, lat)



import pytest
import stripy

# These tests just test the mesh construction doesn't fail at the moment
# ToDo: add meaningful assertions

# It's hard to parameterise these as the calls are a little
# bit different in each case


def test_warped_xy_mesh_sphere():

    mesh = stripy.hybrid_st_meshes.warped_xy_mesh_sphere(res_lon=93, res_lat=28, epsilon=0.001)

    r_array = mesh.XX**2 + mesh.YY**2 + mesh.ZZ**2

    assert r_array.min() > 0.99999
    assert r_array.max() < 1.00001



import pytest
import stripy

# These tests just test the mesh construction doesn't fail at the moment
# ToDo: add meaningful assertions

# It's hard to parameterise these as the calls are a little
# bit different in each case

def test_kd_tree_building_cartesian():
    mesh = stripy.cartesian_meshes.elliptical_mesh((-1.0,1.0,-1.0,1.0), 0.1, 0.2,
                                                    random_scale=0.01, refinement_levels=0, tree=True)

def test_cart_square_border():
    mesh = stripy.cartesian_meshes.square_border((-1.0,1.0,-1.0,1.0), 0.1, 0.1, refinement_levels=0)
    mesh = stripy.cartesian_meshes.square_border((-1.0,1.0,-1.0,1.0), 0.1, 0.1, refinement_levels=2)
    mesh = stripy.cartesian_meshes.square_border((-1.0,1.0,-1.0,1.0), 0.1, 0.2, refinement_levels=0)


def test_elliptical_mesh():

    mesh = stripy.cartesian_meshes.elliptical_mesh((-1.0,1.0,-1.0,1.0), 0.1, 0.1, refinement_levels=0)
    mesh = stripy.cartesian_meshes.elliptical_mesh((-1.0,1.0,-1.0,1.0), 0.1, 0.1, refinement_levels=2)
    mesh = stripy.cartesian_meshes.elliptical_mesh((-1.0,1.0,-1.0,1.0), 0.1, 0.2, refinement_levels=0)
    mesh = stripy.cartesian_meshes.elliptical_mesh((-1.0,1.0,-1.0,1.0), 0.1, 0.1, random_scale=0.01, refinement_levels=0)
    mesh = stripy.cartesian_meshes.elliptical_mesh((-1.0,1.0,-1.0,1.0), 0.1, 0.2, random_scale=0.01, refinement_levels=0)


def test_cart_square_mesh():
    mesh = stripy.cartesian_meshes.square_mesh((-1.0,1.0,-1.0,1.0), 0.1, 0.1, refinement_levels=0)
    mesh = stripy.cartesian_meshes.square_mesh((-1.0,1.0,-1.0,1.0), 0.1, 0.1, refinement_levels=2)
    mesh = stripy.cartesian_meshes.square_mesh((-1.0,1.0,-1.0,1.0), 0.1, 0.2, refinement_levels=0)


def test_cart_random_mesh():
    mesh = stripy.cartesian_meshes.random_mesh((-1.0,1.0,-1.0,1.0), 5000)
    mesh = stripy.cartesian_meshes.random_mesh((-1.0,1.0,-1.0,1.0), 10000)
    mesh = stripy.cartesian_meshes.random_mesh((-1.0,1.0,-1.0,1.0), 20000)
